#include <avr/io.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <util/delay.h>

// RFM12
#define RF12TxBDW(kfrq)	((unsigned char)(kfrq/15)-1)
#define RF12FREQ(freq)	((freq-430.0)/0.0025)	

unsigned short rf12_trans(unsigned short wert)
{
	unsigned short werti = 0;	
	PORTB &= ~(1<<PB2);
	SPDR = (0xFF00 & wert)>>8;
	while(!(SPSR & (1<<SPIF))){};
	werti = (SPDR<<8);
	SPDR = (0x00ff & wert);
	while(!(SPSR & (1<<SPIF))){};
	werti = werti + SPDR;
	PORTB |= (1<<PB2);
	return werti;
}

void rf12_setbandwidth(unsigned char bandwidth, unsigned char gain, 
						unsigned char drssi)
{
	rf12_trans(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void rf12_setfreq(unsigned short freq)
{	
	if (freq<96)					// 430,2400MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz
		freq=3903;
	rf12_trans(0xA000|freq);
}

void rf12_setbaud(unsigned short baud)
{
	if (baud<663)
		return;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		rf12_trans(0xC680|((43104/baud)-1));
	else
		rf12_trans(0xC600|((344828UL/baud)-1));
}

void rf12_setpower(unsigned char power, unsigned char mod)
{	
	rf12_trans(0x9800|(power&7)|((mod&15)<<4));
}

void rf12_ready(void)
//##############################################################################
{
	//unsigned long timeout = 0;
	PORTB &= ~(1<<PB2);
	asm("nop");
	asm("nop");
	asm("nop");
	//wait until FIFO ready or timeout
	while (!(PINB & (1<<PB4)))
	{
		//if ((timeout++)>100000){break;};
	};
	PORTB |= (1<<PB2);
}

void rf12_txdata(unsigned char *data, unsigned char number)
{	
	unsigned char i;
	uint8_t crc = 0;
	rf12_trans(0x8238);			// TX on
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB82D);
	rf12_ready();
	rf12_trans(0xB8D4);
	rf12_ready();
	rf12_trans(0xB800 | (number + 2));	
	crc = crc % (number + 2);
	for (i=0; i<number; i++)
	{	
		rf12_ready();
		rf12_trans(0xB800 | data[i]);
		crc = crc % data[i];
	}
	rf12_ready();
	rf12_trans(0xB800 | crc);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();

	rf12_trans(0x8208);			// TX off
}

unsigned char rf12_rxdata(unsigned char *data)
{	
	unsigned char i, number;
	uint8_t crc = 0;
	uint8_t crc_chk = 0;
	rf12_trans(0x82C8);			// RX on
	rf12_trans(0xCA81);			// set FIFO mode
	rf12_trans(0xCA83);			// enable FIFO
	
	rf12_ready();
	number = rf12_trans(0xB000) & 0x00FF;
	crc_chk = crc_chk % number;
	for (i=0; i<(number - 2); i++)
	{
		rf12_ready();
		data[i] = (unsigned char) (rf12_trans(0xB000) & 0x00FF);
		crc_chk = crc_chk % data[i];
	}
	rf12_ready();
	crc = rf12_trans(0xB000) & 0x00FF;
	rf12_trans(0x8208);			// RX off

	if (crc != crc_chk)
		number = 0;

	return (number - 2);
}


void SetUpRFM12()
{
	DDRB |= (1<<DDB2) | (1<<DDB3) | (1<<DDB5);
	DDRB &= ~(1<<DDB4);
    SPCR = (0 << SPIE) | /* SPI Interrupt Enable */
           (1 << SPE)  | /* SPI Enable */
           (0 << DORD) | /* Data Order: MSB first */
           (1 << MSTR) | /* Master mode */
           (0 << CPOL) | /* Clock Polarity: SCK low when idle */
           (0 << CPHA) | /* Clock Phase: sample on rising SCK edge */
           (1 << SPR1) | /* Clock Frequency: f_OSC / 128 */
           (1 << SPR0);
    SPSR &= ~(1 << SPI2X); /* No doubled clock frequency */
	uint8_t i = 0;
	for (i=0; i<50; i++)
		_delay_ms(10);			// wait until POR done

	rf12_trans(0xC0E0);			// AVR CLK: 10MHz
	rf12_trans(0x80D7);			// Enable FIFO
	rf12_trans(0xC2AB);			// Data Filter: internal
	rf12_trans(0xCA81);			// Set FIFO mode
	rf12_trans(0xE000);			// disable wakeuptimer
	rf12_trans(0xC800);			// disable low duty cycle
	rf12_trans(0xC4F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
	rf12_trans(0x0000);

	for (i=0; i<5; i++)
		_delay_ms(10);			

	rf12_setfreq(RF12FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(5, 1, 4);		// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	rf12_setbaud(9600);				// 9600 baud
	rf12_setpower(0, 6);			// 1mW Ausgangangsleistung, 120k

}


void blink(void)
{
	unsigned short c0, c1;
	unsigned short c0max, c1max;
	c0max = 255;
	c1max = 255;
	PORTC &= ~(1<<PC5);
	for(c0 = 0; c0 < c0max; c0++)
	{
		for(c1 = 0; c1 < c1max; c1++)
		{
			asm volatile ("nop");
		}
	}
	PORTC |= (1<<PC5);
	for(c0 = 0; c0 < c0max; c0++)
	{
		for(c1 = 0; c1 < c1max; c1++)
		{
			asm volatile ("nop");
		}
	}
}

int main(void)
{
	DDRC |= (1<<DDC5);
	PORTC |= (1<<PC5);

	uint8_t i = 0;
	for (i = 0; i < 10; i++)
	{
		blink();
	}
	PORTC |= (1<<PC5);

	SetUpRFM12();

	for(;;)
	{
		blink();
	}
	return 1;
}
